#!/usr/bin/python
import math
import pdb
from OpenGL.GL import *
from OpenGL.GLU import *
from PyQt4 import QtGui
from PyQt4.QtOpenGL import *
from PyQt4.Qt import Qt
from PyQt4 import QtCore
from PyNGL import *
import sys
import random
import pdb
import time


class SpotData :
		def __init__(self):
			rand=Random.instance();
			# @brief the time offset for the spot
			self.m_time=rand.randomPositiveNumber(3)+0.6
			# @brief x radius of ellipse
			self.m_radiusX=rand.randomNumber(1)+0.5
			# @brief z radius of the ellipse
			self.m_radiusZ=rand.randomNumber(1)+0.5
			# @brief current mix factor of the light for Lerp of colours
			self.m_mix=0.0
			# @brief aim center of the spot where the ellipse is centered
			self.m_aimCenter=rand.getRandomPoint(4,0,14)
			# @brief start colour of light interpolated to end colour
			self.m_startColour=rand.getRandomColour()
			# @brief end colour of the light
			self.m_endColour=rand.getRandomColour()
			self.m_startColour.clamp(0.2,1.0)
			self.m_endColour.clamp(0.2,1.0)
			self.m_animate=True


class GLWindow(QGLWidget):

	def __init__(self, parent):
		QGLWidget.__init__(self, parent)
		self.setMinimumSize(1024, 720)
		self.m_spinYFace = 0
		self.m_spinXFace = 0
		self.m_origX = 0
		self.m_origY = 0
		self.m_transformStack=TransformStack()
		self.CurrentCamera=0
		self.m_lightChangeTimer=self.startTimer(100);
		self.m_lightParamChangeTimer=self.startTimer(1800);
		self.m_spots=[]
		self.m_spotData=[]
		self.m_time=0
		self.m_animate=True


	def mousePressEvent (self,  _event) :

		# this method is called when the mouse button is pressed in this case we
		# store the value where the maouse was clicked (x,y) and set the Rotate flag to true
		if(_event.button() == Qt.LeftButton) :
			self.m_origX = _event.x();
			self.m_origY = _event.y();
			self.m_rotate =True;



	def mouseMoveEvent ( self,_event ) :
		# note the method buttons() is the button state when event was called
		# this is different from button() which is used to check which button was
		# pressed when the mousePress/Release event is generated

		if(self.m_rotate and _event.buttons() == Qt.LeftButton) :
			self.m_spinYFace = ( self.m_spinYFace + (_event.x() - self.m_origX) ) % 360
			self.m_spinXFace = ( self.m_spinXFace + (_event.y() - self.m_origY) ) % 360
			self.m_origX = _event.x();
			self.m_origY = _event.y();
			# re-draw GL
			self.updateGL();

	def mouseReleaseEvent (self,  _event) :

		# this event is called when the mouse button is released
		# we then set Rotate to false
		if (_event.button() == Qt.LeftButton) :
			self.m_rotate=False



	def createLights(self) :
		rand=Random.instance();
		x=0.0;
		z=0.0;
		for i in range(0,7) :

			x=rand.randomNumber(4);
			z=rand.randomNumber(4);
			pos=Vector(x,4,z);
			dir=Vector(0,-1,0);
			c=rand.getRandomColour();
			c.clamp(0.5,1.0);
			self.m_spotData.append(SpotData())

			self.m_spots.append(SpotLight(pos,dir,c));
		for i in range(0,7) :
			self.m_spots[i].setSpecColour(Colour(1,1,1,1));
			self.m_spots[i].setParams(rand.randomPositiveNumber(10)+1,rand.randomPositiveNumber(2)+1,0.4,0,0);



	def paintGL(self):
			'''
			Drawing routine
			'''
			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
			trans=Transformation()
			# set the mouse rotation
			trans.setRotation(self.m_spinXFace,self.m_spinYFace,0)
			# set this in the TX stack
			self.m_transformStack.setGlobal(trans);
			for i in range(0,len(self.m_spots)) :
				self.m_spots[i].enable()
			# Rotation based on the mouse position for our global
			# transform
			self.shader=ShaderManager.instance()

			self.shader.setShaderParamFromMatrix("gl3xTest","ViewMatrix",self.m_cam.getModelView())
			self.shader.setShaderParamFromMatrix("gl3xTest","projectionMatrix",self.m_cam.getProjection())
			self.m_transformStack.loadGlobalAndCurrentMatrixToShader("gl3xTest","ModelMatrix",ACTIVEMATRIX.NORMAL);

			prim=VBOPrimitives.instance()

			prim.draw("plane")
			for z in range(-4,5,2) :
				for x in range(-4,5,2) :
					self.m_transformStack.pushTransform();

					self.m_transformStack.getCurrentTransform().setRotation(0,(x*z)*20,0);

					self.m_transformStack.getCurrentTransform().setPosition(Vector(x,0.49,z));
					self.m_transformStack.loadGlobalAndCurrentMatrixToShader("gl3xTest","ModelMatrix",ACTIVEMATRIX.NORMAL);

					prim.draw("teapot");
					self.m_transformStack.popTransform();

			glFlush()


	def resizeGL(self, w, h):
			'''
			Resize the GL window
			'''
			glViewport(0, 0, w, h)
			self.m_cam.setShape(45,float(w/h),0.5,10,CAMERAPROJECTION.PERSPECTIVE)



	def initializeGL(self):
			'''
			Initialize GL
			'''
			# set viewing projection
			glShadeModel(GL_SMOOTH)
			glEnable(GL_DEPTH_TEST)

			glClearColor(0.0, 0.0, 0.0, 1.0)
			glClearDepth(1.0)
			From=Vector(10,4,2)
			To=Vector(0,0,0)
			Up=Vector(0,1,0)
			self.m_cam=Camera(From,To,Up,CAMERAPROJECTION.PERSPECTIVE)
			self.m_cam.setShape(45,1024.0/720.0,0.5,10,CAMERAPROJECTION.PERSPECTIVE);
			#pdb.set_trace()

			self.shader=ShaderManager.instance()
			# load a frag and vert shaders
			self.shader.loadShader("gl3xTest","Vertex.vs","Fragment.fs","",True)
			# set this as the active shader
			self.shader.useShader("gl3xTest");
			# now pass the modelView and projection values to the shader
			self.shader.setShaderParamFromMatrix("gl3xTest","ViewMatrix",self.m_cam.getModelView())
			self.shader.setShaderParamFromMatrix("gl3xTest","projectionMatrix",self.m_cam.getProjection())

			self.shader.setShaderParam1i("gl3xTest","Normalize",True);
			self.shader.setShaderParam1i("gl3xTest","numLightsEnabled",8);
			self.shader.setShaderParam4f("gl3xTest","eye",10,4,2,1);

			prim =VBOPrimitives.instance();
			prim.createVBOQuadPlane("plane",20,20,280,280,Vector(0,1,0));

			self.createLights()

			m=Material(STDMAT.POLISHEDSILVER)
			m.use()

	def timerEvent(self, _event) :
		timer=_event.timerId()
		if _event.timerId() == self.m_lightChangeTimer :
			if self.m_animate !=True :
				return;

			for i in range(0,len(self.m_spots) ):
				pointOnCircleX= math.cos(self.m_time+self.m_spotData[i].m_time )*self.m_spotData[i].m_radiusX;
				pointOnCircleZ= math.sin(self.m_time+self.m_spotData[i].m_time)*self.m_spotData[i].m_radiusZ;
				p=Vector(self.m_spotData[i].m_aimCenter[0]+pointOnCircleX,0,self.m_spotData[i].m_aimCenter[2]+pointOnCircleZ);

				#self.m_spots[i].setColour(lerp(self.m_spotData[i].m_startColour,m_spotData[i].m_endColour,m_spotData[i].m_mix ));
				self.m_spotData[i].m_mix+=0.05;
				if self.m_spotData[i].m_mix >1.0 :
					self.m_spotData[i].m_mix=0.0;
				self.m_spots[i].aim(p);

				self.m_time+=0.01;
			self.updateGL();

	def changeSpotParams(self) :
		rand=Random.instance();
		rand.setSeed(int(time.time()))
		for i in range(0,len(self.m_spotData)) :
			self.m_spotData[i].m_aimCenter=rand.getRandomPoint(4,0,14);
			self.m_spotData[i].m_radiusX=rand.randomNumber(2)+0.5;
			self.m_spotData[i].m_radiusZ=rand.randomNumber(2)+0.5;
			self.m_spotData[i].m_time=rand.randomPositiveNumber(4)+0.6;
			self.m_spotData[i].m_startColour =rand.getRandomColour();
			self.m_spotData[i].m_endColour =rand.getRandomColour();
			self.m_spotData[i].m_mix=0.0;


# You don't need anything below this
class NGLOpenGLDemo(QtGui.QMainWindow):

	def __init__(self):
		QtGui.QMainWindow.__init__(self)
		self.widget = GLWindow(self)
		self.setCentralWidget(self.widget)


	def keyPressEvent(self ,_event) :
		INC=0.1

		if _event.key() == Qt.Key_Q	:
			sys.exit()
		elif _event.key() == Qt.Key_W :
			glPolygonMode(GL_FRONT_AND_BACK,GL_LINE)
		elif _event.key() == Qt.Key_S :
			glPolygonMode(GL_FRONT_AND_BACK,GL_FILL)
		elif _event.key() == Qt.Key_Up :
			self.widget.m_cam.move(0.0,-INC,0.0)
		elif _event.key() == Qt.Key_Down :
			self.widget.m_cam.move(0.0,INC,0.0)
		elif _event.key() == Qt.Key_Left :
			self.widget.m_cam.move(INC,0.0,0.0)
		elif _event.key() == Qt.Key_Right :
			self.widget.m_cam.move(-INC,0.0,0.0)

		elif _event.key() == Qt.Key_I :
			self.widget.m_cam.move(0.0,0.0,INC)
		elif _event.key() == Qt.Key_O :
			self.widget.m_cam.move(0.0,0.0,-INC)
		elif _event.key() == Qt.Key_A :
			self.widget.m_animate^=True
		elif _event.key() == Qt.Key_Space :
			self.widget.changeSpotParams()

		self.widget.updateGL()


if __name__ == '__main__':
	app = QtGui.QApplication(['Light Demo'])
	window = NGLOpenGLDemo()
	window.setWindowTitle("PyNGL Light Demo")
	window.show()
	app.exec_()